﻿# -*- coding: utf-8 -*-
"""
/***************************************************************************
 gpsUtils
                                 A QGIS plugin
                              -------------------
        begin                : 2013-04-06
        copyright            : (C) 2013 by Piotr Pociask
        email                : info@gis-support.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt4.QtGui import *
from PyQt4.QtCore import *
from qgis.core import *
from qgis.gui import *
from math import sqrt
from time import strftime
from os import remove, stat
import GPSTrackerDialog
try:
    from winsound import Beep
except ImportError:
    def playsound(frequency,duration):
        pass
else: 
    def playsound(frequency,duration):
        Beep(frequency,duration)

#import sys
#sys.path.append('C:\Users\Pocisk\.eclipse\org.eclipse.platform_4.3.0_578536936_win32_win32_x86_64\plugins\org.python.pydev_2.8.1.2013072611\pysrc')
#from pydevd import settrace

import functools, traceback, sys

"""=============Obsługa wyjątków============"""
def catch_exception(f):
    @functools.wraps(f)
    def func(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except:
            exc_type, value, tb = sys.exc_info()
            GPSTrackerDialog.GPSLogger.writeException('%s' % (''.join([x.replace('\n', '\r\n') for x in traceback.format_exception(exc_type, value, tb)])))
    return func

class ErrorCatcher(pyqtWrapperType):
    def __new__(cls, name, bases, dct):
        for m in dct:
            if hasattr(dct[m], '__call__'):
                if type(dct[m]) is pyqtSignal:
                    continue
                dct[m] = catch_exception(dct[m])
        return pyqtWrapperType.__new__(cls, name, bases, dct)

"""=============Elementy okna mapy============"""
class GPSMarkerItem(QgsMapCanvasItem):
    __metaclass__ = ErrorCatcher
    
    def __init__(self, canvas, parent=None):
        QgsMapCanvasItem.__init__(self, canvas)
        self.parent = parent
        self.setLabel('')
        self.map_pos = QgsPoint(0.0, 0.0)

    def paint(self, painter, xxx, xxx2):
        self.setPos(self.toCanvasCoordinates(self.map_pos))
        halfSize = self.parent.markerSize / 2.0
        self.parent.markerImg.paint(painter, 0 - halfSize, 0 - halfSize, self.parent.markerSize,
                                    self.parent.markerSize, alignment=Qt.AlignCenter)
        painter.setPen(self.parent.markerFontColor)
        painter.setFont(self.parent.markerFont)
        painter.drawText(halfSize+1, 0, self.label)

    def boundingRect(self):
        halfSize = self.parent.markerSize / 2.0
        if halfSize<self.parent.markerFontMetric.ascent():
            halfSize = self.parent.markerFontMetric.ascent()
        return QRectF(-halfSize, -halfSize, (2.0 * halfSize)+self.labelWidth+1, 2.0 * halfSize)

    def setCenter(self, map_pos):
        self.map_pos = map_pos
        self.setPos(self.toCanvasCoordinates(self.map_pos))
    
    def setLabel(self, label):
        self.label = label
        self.labelWidth = self.parent.markerFontMetric.width(self.label)

    def updatePosition(self):
        self.setCenter(self.map_pos)

class GPSMarker(QObject):
    __metaclass__ = ErrorCatcher
    
    def __init__(self, canvas, iconPath, parent=None):
        QObject.__init__(self, parent)
        self.marker = None
        self.canvas = canvas
        self.model = parent.tvPointList.model()
        self.parent = parent
        self.setMarkerIcon(iconPath)
        self.setMarkerSize(32)
        self.setMarkerFont()
    
    def clean(self):
        self.setMarkerVisible(False)
    
    def setMarkerIcon(self, iconPath):
        self.markerImg = QIcon(iconPath)
    
    def setMarkerSize(self, size=24):
        self.markerSize = size
    
    def setMarkerFont(self, family='MS Shell Dlg 2', color=QColor('black'), size=8):
        self.markerFont = QFont(family, size)
        self.markerFont.setBold(True)
        self.markerFontColor = color
        self.markerFontMetric = QFontMetricsF(self.markerFont)
        if self.marker:
            self.marker.labelWidth = self.markerFontMetric.width(self.marker.label)
        
    def setMarkerPos(self, x, y):
        self.markerPos = self.transform.transform(x, y)
        if self.marker:
            if self.parent.gbPointDistance.isChecked():
                coords = self.model.getLastPoint()
                if coords:
                    lastPoint = self.transform.transform(coords[0], coords[1])
                    distance = sqrt(lastPoint.sqrDist(self.markerPos))
                    self.marker.setLabel('%.2f' % distance)
                else:
                    self.marker.setLabel('')
            else:
                self.marker.setLabel('')
            self.marker.setCenter(self.markerPos)
    
    def setMarkerVisible(self, visible):
        if visible:
            self.marker = GPSMarkerItem(self.canvas, self)
        else:
            self.canvas.scene().removeItem(self.marker)
            self.marker = None

class GPSPath(QObject):
    __metaclass__ = ErrorCatcher
    
    def __init__(self, canvas, parent):
        QObject.__init__(self, parent)
        self.canvas = canvas
        self.parent = parent
        self.path = QgsRubberBand(canvas)
        self.vertexList = []
        self.parent.tvPointList.model().dataChanged.connect(self.dataChanged)
        
    def clean(self):
        self.setVisible(False)
    
    def dataChanged(self, *args):
        self.setVisible(self.parent.cbDisplayPoints.checkState()==Qt.Checked)
    
    def addVertex(self, point):
        vertex = QgsVertexMarker(self.canvas)
        vertex.setCenter(point)
        return vertex
    
    def setVisible(self, visible):
        self.path.reset()
        for point in self.vertexList:
            self.canvas.scene().removeItem(point)
        self.vertexList = []
        if visible:
            points = self.parent.tvPointList.model().getPointList()
            for coords in points:
                point = self.transform.transform(coords['x'], coords['y'])
                self.path.addPoint(point)
                self.vertexList.append(self.addVertex(point))

class GPSSelectedMarker(QObject):
    __metaclass__ = ErrorCatcher
    
    def __init__(self, canvas, parent=None):
        QObject.__init__(self, parent)
        self.marker = None
        self.point = None
        self.canvas = canvas
    
    def clean(self):
        self.setMarker(-1, QgsPoint())

    def createMarker(self):
        self.marker = QgsVertexMarker(self.canvas)
        self.marker.setPenWidth(2)
        self.marker.setColor(QColor('blue'))
        self.marker.setIconType(QgsVertexMarker.ICON_BOX)
    
    def destroyMarker(self):
        self.canvas.scene().removeItem(self.marker)
        self.marker = None
    
    def setMarker(self, index, point):
        self.point = point
        if index == -1:
            self.destroyMarker()
            return
        elif not self.marker:
            self.createMarker()
        self.marker.setCenter(self.transform.transform(point))

"""=============Zapisywanie danych============"""
class GPSDataWriter(QObject):
    __metaclass__ = ErrorCatcher
    wgs84 = QgsCoordinateReferenceSystem(4326, QgsCoordinateReferenceSystem.EpsgCrsId)
    geomIcon = {QGis.Point:'pointLayer.svg', QGis.Line:'lineLayer.svg', QGis.Polygon:'polygonLayer.svg'}
    iconsPath = str(QFileInfo(__file__).absolutePath()) + '/icons/'
    nrFieldIndex = -1
    
    def __init__(self, parent, cmbLayers):
        QObject.__init__(self, parent)
        self.parent = parent
        self.cmbLayers = cmbLayers
        self.parent.connection.gpsMeasureStopped[dict, bool].connect(self.pointReceived)
        QgsMapLayerRegistry.instance().layersAdded.connect(self.addLayers)
        QgsMapLayerRegistry.instance().layersWillBeRemoved[QStringList].connect(self.removeLayers)
        cmbLayers.currentIndexChanged[int].connect(self.changeLayer)
        #jeszcze zmiana nazwy warstwy
        self.setLayer(None)
        self.getAllLayers()
    
    def clean(self):
        QgsMapLayerRegistry.instance().layersAdded.disconnect(self.addLayers)
        QgsMapLayerRegistry.instance().layersWillBeRemoved[QStringList].disconnect(self.removeLayers)
    
    def getAllLayers(self):
        self.addLayers(QgsMapLayerRegistry.instance().mapLayers().itervalues())
    
    def setLayer(self, layer):
        self.activeLayer = layer
        if layer:
            self.transform = QgsCoordinateTransform(self.wgs84, layer.crs())
    
    def changeLayer(self, index):
        layerId = self.sender().itemData(index).toString()
        layer = QgsMapLayerRegistry.instance().mapLayer(layerId)
        if not layer:
            self.setLayer(None)
            return
        if str(layer.name().toUtf8()).lower() == 'punkty_pomocnicze' and layer.geometryType() == QGis.Point:
            self.nrFieldIndex = layer.fieldNameIndex('NR')
        else:
            self.nrFieldIndex = -1
        self.setLayer(layer)
    
    def addLayers(self, layers):
        for layer in layers:
            if layer.type() == QgsMapLayer.VectorLayer:
                icon = QIcon(self.iconsPath + self.geomIcon[layer.geometryType()])
                self.cmbLayers.addItem(icon, layer.name(), layer.id())
        self.cmbLayers.model().sort(0)
    
    def removeLayers(self, layerIds):
        #ta funkcja jest wywoływana podwójnie
        model = self.cmbLayers.model()
        start = model.index(0, 0)
        for layerId in layerIds:
            try:
                row = model.match(start, Qt.UserRole, QVariant(layerId))[0].row()
                model.removeRow(row)
            except:
                continue
    
    def pointReceived(self, coords, updatePoint=False):
        if updatePoint or not self.activeLayer:
            return
        if self.activeLayer.isEditable() and self.activeLayer.geometryType() == QGis.Point:
            self.savePoint(coords, self.parent.tvPointList.model().rowCount(), self.getShowFeatureForm())
            self.parent.iface.mapCanvas().refresh()
    
    def savePoint(self, coords, index, showFeatureForm):
        point = QgsFeature()
        point.setGeometry(QgsGeometry.fromPoint(self.transform.transform(coords['x'], coords['y'])))
        point.setAttributeMap(self.mapAttribute)
        if self.nrFieldIndex != -1:
            point.changeAttribute(self.nrFieldIndex, QVariant(index+1))
            showFeatureForm = False
        self.activeLayer.addFeature(point)
        if showFeatureForm and self.isFirstPoint:
            self.parent.iface.openFeatureForm(self.activeLayer, point)
            self.mapAttribute = point.attributeMap()
    
    def savePoints(self, allPoints):
        self.isFirstPoint = True
        self.mapAttribute = {x:None for x in range(self.activeLayer.dataProvider().fieldCount())}
        self.activeLayer.startEditing()
        points = self.parent.tvPointList.model().getPointList(allPoints)
        showFeatureForm = self.getShowFeatureForm()
        for i, point in enumerate(points):
            self.savePoint(point, i, showFeatureForm)
            self.isFirstPoint = False
        self.activeLayer.commitChanges()
        self.parent.iface.mapCanvas().refresh()
    
    @pyqtSlot()
    def saveFeature(self):
        if self.parent.tvPointList.model().rowCount() == 0:
            QMessageBox.critical(None, 'GPS Tracker Plugin', u'Brak zarejestrowanych punktów!')
            return
        allPoints = self.sender().allPoints
        geomType = self.activeLayer.geometryType()
        if geomType == QGis.Point:
            self.savePoints(allPoints)
        else:
            self.saveObject(allPoints, geomType)
    
    def saveObject(self, checkedOnly, geomType):
        points = self.parent.tvPointList.model().getPointList(checkedOnly)
        if (geomType == QGis.Line and len(points) < 2) or (geomType == QGis.Polygon and len(points) < 3):
            QMessageBox.critical(None, 'GPS Tracker Plugin', u'Liczba zarejestrowanych lub wybranych punktów jest zbyt mała!')
            return
        showFeatureForm = self.getShowFeatureForm()
        self.activeLayer.startEditing()
        feat = QgsFeature()
        if geomType == QGis.Line:
            feat.setGeometry(QgsGeometry.fromPolyline([self.transform.transform(point['x'], point['y']) for point in points]))
        else:
            feat.setGeometry(QgsGeometry.fromPolygon([[self.transform.transform(point['x'], point['y']) for point in points]]))
        for i, field in enumerate(self.activeLayer.pendingFields().keys()):
            feat.addAttribute(i, QVariant(None))
        self.activeLayer.addFeature(feat)
        if showFeatureForm:
            self.parent.iface.openFeatureForm(self.activeLayer, feat)
        self.activeLayer.commitChanges()
        self.parent.iface.mapCanvas().refresh()
    
    @staticmethod
    def getShowFeatureForm():
        return not QSettings().value('Qgis/digitizing/disable_enter_attribute_values_dialog').toBool()

class GPSLogFile(QObject):
    ''' Pojedynczy plik logowania '''
    __metaclass__ = ErrorCatcher
    
    def __init__(self, filePath, parent=None):
        QObject.__init__(self, parent)
        self.filePath = filePath
        self.openFile()
    
    def clean(self):
        self.closeFile()
    
    def openFile(self):
        self.logFile = open(self.filePath, 'a')
        
    def writeData(self, data):
        self.logFile.write('%s' % data)
        self.logFile.write('\r\n')
    
    def closeFile(self):
        self.logFile.close()

class GPSLogger(QObject):
    __metaclass__ = ErrorCatcher
    ''' Klasa zbiorcza do logowania '''
    def __init__(self, parent=None):
        QObject.__init__(self, parent)
        self.parent = parent
        self.changeDirectory(parent.eLogDir.text())
        self.writeNmea = False
        self.nmeaLog = None
        self.parent.connection.gpsMeasureStopped[dict, bool].connect(self.writeMeasuredPoint)
    
    def clean(self):
        self.parent.connection.gpsMeasureStopped[dict, bool].disconnect(self.writeMeasuredPoint)
        if self.nmeaLog:
            self.nmeaLog.closeFile()
    
    def openFile(self):
        self.setSessionDateTime()
        self.nmeaFile = '%s\%s.nmea' % (self.logDirectory, self.sessionDateTime)
        self.nmeaLog = GPSLogFile(self.nmeaFile)
        self.parent.connection.connection.nmeaSentenceReceived[str].connect(self.nmeaReceived)
    
    def closeFile(self):
        self.parent.connection.connection.nmeaSentenceReceived[str].disconnect(self.nmeaReceived)
        self.nmeaLog.closeFile()
        if not stat(self.nmeaFile).st_size:
            remove(self.nmeaFile)
        self.nmeaLog = None
    
    def setSessionDateTime(self):
        #wywoływane w momencie połączenia
        self.sessionDateTime = self.getDateTimeFormat()
    
    def nmeaReceived(self, nmea):
        if self.writeNmea:
            nmea = nmea.trimmed()
            self.nmeaLog.writeData(nmea)
    
    def writeMeasuredPoint(self, calcPoint, updatePoint):
        #zapidywanie danych z pomiarów
        measureLog = GPSLogFile('%s\pomiar_%s.log' % (self.logDirectory, self.sessionDateTime))
        try:
            measureLog.writeData('Pomiar z %s:' % (strftime('%H:%M:%S %d.%m.%Y')))
            for coords in self.sender().measuredPoints:
                point = self.transform.transform(coords[0], coords[1])
                measureLog.writeData('%.10f %.10f' % (point.x(), point.y()))
            point = self.transform.transform(calcPoint['x'], calcPoint['y'])
            measureLog.writeData('\r\nUśredniony wynik z %d pomiarów:\r\n%.10f %.10f\r\n' % (calcPoint['lp'], point.x(), point.y()))
            measureLog.writeData('===================================\r\n')
        finally:
            measureLog.closeFile()
    
    @classmethod
    def changeDirectory(cls, directory):
        cls.logDirectory = str(directory)
    
    @classmethod
    def writeException(cls, text):
        errorFile = 'error_%s.log' % cls.getDateTimeFormat()
        errorLog = GPSLogFile('%s\\%s' % (cls.logDirectory, errorFile))
        try:
            errorLog.writeData(text)
        finally:
            errorLog.closeFile()
            QMessageBox.critical(None, '',
                                 u"Wystąpił błąd podczas wykonywania operacji!\n\
Dane zostały zapisane w pliku '%s' w katalogu logowania." % errorFile)
    
    @staticmethod
    def getDateTimeFormat():
        return strftime('%Y%m%d_%H%M%S')
    
    @staticmethod
    def verifyDirectory(dir):
        if not dir or not QDir(dir).exists():
            dir = '%s/log' % QFileInfo(__file__).absolutePath()
        return dir

"""=============Wcięcie liniowe============"""
class GPSResection(QObject):
    __metaclass__ = ErrorCatcher
    wgs84 = QgsCoordinateReferenceSystem(4326, QgsCoordinateReferenceSystem.EpsgCrsId)
    gpsDataChanged = pyqtSignal(QgsPoint)
    
    def __init__(self, parent=None):
        QObject.__init__(self, parent)
        self.parent = parent
        self.canvas = self.parent.iface.mapCanvas()
        self.leftPoint = None
        self.rightPoint = None
        self.rightDistance = 0.
        self.leftDistance = 0.
        self.leftMarker = None
        self.rightMarker = None
        self.calcMarker = None
    
    @pyqtSlot()
    def clean(self):
        self.leftPoint = None
        self.rightPoint = None
        self.rightDistance = 0.
        self.leftDistance = 0.
        self.leftMarker = self.showMarker(self.leftMarker, False)
        self.rightMarker = self.showMarker(self.rightMarker, False)
        self.calcMarker = self.showMarker(self.calcMarker, False)
        self.parent.lblAX.setText('N/A')
        self.parent.lblAY.setText('N/A')
        self.parent.lblBX.setText('N/A')
        self.parent.lblBY.setText('N/A')
        self.parent.lblXP.setText('N/A')
        self.parent.lblYP.setText('N/A')
        self.parent.sbAP.setValue(0.)
        self.parent.sbBP.setValue(0.)
    
    @pyqtSlot()
    def addCalcPoint(self):
        calcPoint = self.calcResection()
        if calcPoint:
            transform = QgsCoordinateTransform(self.canvas.mapRenderer().destinationCrs(), self.wgs84)
            calcPoint84 = transform.transform(calcPoint)
            pointData = {'x':calcPoint84.x(), 'y':calcPoint84.y(), 'lp':0}
            self.parent.tvPointList.model().insertRow(pointData, False)
    
    def setLeftPoint(self, point):
        self.leftPoint = point
        self.leftMarker = self.showMarker(self.leftMarker, True, QColor('red'), self.leftPoint)
        self.parent.lblAX.setText(str(self.leftPoint.x()))
        self.parent.lblAY.setText(str(self.leftPoint.y()))
        self.calcResection()
    
    def setRightPoint(self, point):
        self.rightPoint = point
        self.rightMarker = self.showMarker(self.rightMarker, True, QColor('blue'), self.rightPoint)
        self.parent.lblBX.setText(str(self.rightPoint.x()))
        self.parent.lblBY.setText(str(self.rightPoint.y()))
        self.calcResection()
    
    @pyqtSlot()
    def reversePoints(self):
        leftPoint = self.leftPoint
        self.leftPoint = self.rightPoint
        self.rightPoint = leftPoint
        self.calcResection()
        self.setMarkersVisible(2)
    
    def setLeftDistance(self, distance):
        self.leftDistance = distance
        self.calcResection()
    
    def setRightDistance(self, distance):
        self.rightDistance = distance
        self.calcResection()
    
    def calcResection(self):
        try:
            c = sqrt(self.leftPoint.sqrDist(self.rightPoint))
            p = self.rightDistance
            l = self.leftDistance
            Xl = self.leftPoint.x()
            Yl = self.leftPoint.y()
            Xp = self.rightPoint.x()
            Yp = self.rightPoint.y()
            Ca = -p**2+l**2+c**2
            Cb = p**2-l**2+c**2
            Cc = p**2+l**2-c**2
            P4 = sqrt(Ca*Cb+Ca*Cc+Cb*Cc)
            Xw = (Xl*Cb+Yl*P4+Xp*Ca-Yp*P4)/(Ca+Cb)
            Yw = (-P4*Xl+Yl*Cb+P4*Xp+Yp*Ca)/(Ca+Cb)
        except:
            self.parent.lblXP.setText('N/A')
            self.parent.lblYP.setText('N/A')
            self.calcMarker = self.showMarker(self.calcMarker, False)
            return None
        else:
            self.parent.lblXP.setText(str(Xw))
            self.parent.lblYP.setText(str(Yw))
            calcPoint = QgsPoint(Xw, Yw)
            self.calcMarker = self.showMarker(self.calcMarker, True, QColor('green'),  calcPoint)
            return calcPoint
    
    def setMarkersVisible(self, index):
        if index != 2:
            if self.canvas.mapTool() == self.parent.getLeftPoint:
                self.canvas.unsetMapTool(self.parent.getLeftPoint)
            if self.canvas.mapTool() == self.parent.getRightPoint:
                self.canvas.unsetMapTool(self.parent.getRightPoint)
            self.calcMarker = self.showMarker(self.calcMarker, False)
        else:
            self.calcResection()
        self.leftMarker = self.showMarker(self.leftMarker, index==2 and self.leftPoint, QColor('red'), self.leftPoint)
        self.rightMarker = self.showMarker(self.rightMarker, index==2 and self.rightPoint, QColor('blue'), self.rightPoint)
    
    def showMarker(self, marker, visible, color=QColor('green'), point=None):
        if visible:
            if not marker:
                marker = self.createMarker(color)
            marker.setCenter(point)
        elif marker:
            self.canvas.scene().removeItem(marker)
            marker = None
        return marker
    
    def createMarker(self, color):
        marker = QgsVertexMarker(self.canvas)
        marker.setColor(color)
        return marker

class GPSGetCanvasPoint(QgsMapTool):
    __metaclass__ = ErrorCatcher
    emitPoint = pyqtSignal(QgsPoint)
    
    def __init__(self, canvas, btn):
        QgsMapTool.__init__(self, canvas)
        self.setButton(btn)
        self.setSignals()
    
    def setSignals(self):
        self.mButton.toggled[bool].connect(self.setMapTool)
    
    def setMapTool(self, setTool):
        if setTool:
            self.canvas().setMapTool(self)
        else:
            self.canvas().unsetMapTool(self)
    
    def canvasReleaseEvent(self, e):
        p = self.toMapCoordinates(QPoint(e.x(), e.y()))
        self.emitPoint.emit(p)
    
    def setButton(self, btn):
        self.mButton = btn
    
    def button(self):
        return self.mButton
    
    def activate(self):
        if self.mButton:
            self.mButton.setChecked(True)
        QgsMapTool.activate(self)
    
    def deactivate(self):
        try:
            if self.mButton:
                self.mButton.setChecked(False)
            QgsMapTool.deactivate(self)
        except:
            pass
    
    def isZoomTool(self):
        return False

    def isTransient(self):
        return False

    def isEditTool(self):
        return False